#ifndef _PX_BODY_H_
#define _PX_BODY_H_

class pxBody
{
public:
	int _body;
public:
	pxBody();
	void Delete();
	void CreatePlane(float x, float y, float z);
	void CreateCube(float dx, float dy, float dz, float mass);
	void CreateSphere(float radius, float mass);
	void CreateCapsule(float height, float radius, float mass);
	void CreateCylinder(float radius, float height, int nbEdge, float mass);
	void CreateHull(float* vbank, int nvert, float mass);

	void CreateTriMesh(float* vbank, int* fbank, int MESH_NBVERTICES, int MESH_NBFACES, float mass);
	void CreateTerrain(int nSize, int bank, float scale_x, float scale_y, float scale_z);
	void CreateTerrainFromMesh(int vbank, int fbank, int MESH_NBVERTICES, int MESH_NBFACES, float axis);
	void CreateTriMeshToFile(int vbank, int fbank, int MESH_NBVERTICES, int MESH_NBFACES, char* fname);
	void CreateTriMeshFromFile(char* fname, float mass);


	void SetPosition(float pos_x, float pos_y, float pos_z);
	void SetRotation(float pitch, float yaw, float roll);

	Vector3 GetPosition();
	Vector3 GetRotation();


	void SetFrozen(int stat);
	void SetFrozenRot(int x, int y, int z);
	void SetFrozenPos(int x, int y, int z);
	void FlagGravity(int stat);
	void FlagCollision(int stat);
	void FlagResponse(int stat);
	void FlagContacttable(int stat);
	void FlagMagniteble(int stat);
	void FlagTriggertable(int stat);
	void FlagRayCast(int stat);

	void AddForce(Vector3 vec, int mode);
	void AddForceAtPos(Vector3 vec, Vector3 point, int mode);
	void AddForceAtLocalPos(Vector3 vec, Vector3 point, int mode);
	void AddLocalForce(Vector3 vec, int mode);
	void AddLocalForceAtPos(Vector3 vec, Vector3 point, int mode);
	void AddLocalForceAtLocalPos(Vector3 vec, Vector3 point, int mode);

	void	AddTorque(Vector3 torq, int mode);
	void	AddLocalTorque(Vector3 torq, int mode);
	void	SetAngularSpeed(Vector3 speed);
	void	SetLinearSpeed(Vector3 speed);
	void	SetLocalLinearSpeed(Vector3 speed);
	void	SetLocalAngularSpeed(Vector3 speed);
	void	SetAngularDamping(float angDamp);
	void	SetLinearDamping(float linDamp);
	void	SetAngularMomentum(Vector3 mom);
	void	SetLinearMomentum(Vector3 mom);
	void	SetMaxAngularSpeed(float tres);
	void	SetSleepEnergyThreshold(float tres);
	void	SetSleepAngularVelocity(float tres);
	void	SetSleepLinearVelocity(float tres);
	void	SetSleepWakeUp(float wakeCounterValue);
	void	SetSleepPut();
	void	SetSolverIterationCount(int itercount);
	void	SetBodyName(char* name);
	void	SetBodyEntity(int entity);
	void	SetBodyUserData(int userdata);
	void	MoveBodyToPoint(float maxspeed, float x, float y, float z);
	void	AddBodytoBody(int body);
	int		CopyBody();

	float	GetAngularSpeedModul();
	Vector3 GetAngularSpeed();
	Vector3 GetLocalAngularSpeed();
	float	GetLinearSpeedModul();
	Vector3 GetLinearSpeed();
	Vector3 GetLocalLinearSpeed();
	Vector3 GetLinearVecSpeed();
	float	GetLocalPointSpeedModul(Vector3 point);
	Vector3 GetLocalPointSpeed(Vector3 point);

	float	GetAngularDamping();
	float	GetLinearDamping();
	float	GetAngularMomentum();
	float	GetLinearMomentum();
	float	GetMaxAngularVelocity();
	char*	GetName();
	int		GetEntity();
	int		GetUserData();
	float	GetSleepAngularVelocity();
	float	GetSleepLinearVelocity();
	int		IsDynamic();
	int		IsSleeping();

	void	SetMass(float mass);
	void	SetCMassLocalPosition(Vector3 pos);
	void	SetCMassGlobalPosition(Vector3 pos);
	void	SetMassSpaceInertiaTensor(Vector3 pos);
	Vector3 GetCMassGlobalPosition();
	Vector3 GetCMassLocalPosition();
	float	GetMass();

	int		Contacts();
	int		ContactGetBody(int coll);
	Vector3 ContactGetPoint(int coll);
	Vector3 ContactGetPointN(int coll);
	float	ContactGetForceNModul(int coll);
	Vector3 ContactGetForceN(int coll);
	float	ContactGetForceTModul(int coll);
	Vector3 ContactGetForceT(int coll);
	int		ContactEventsOnStartTouch();
	int		ContactEventsOnEndTouch();
};

pxBody::pxBody()
{
	_body = NULL;
}
void pxBody::Delete()
{
	pxDeleteBody(_body);
}

void pxBody::CreatePlane(float x = 0, float y = 0, float z = 0)
{
	_body = pxBodyCreatePlane(x, y, z);
}
void pxBody::CreateCube(float dx = 1, float dy = 1, float dz = 1, float mass = 10)
{
	_body = pxBodyCreateCube(dx, dy, dz, mass);
}
void pxBody::CreateSphere(float radius, float mass = 10)
{
	_body = pxBodyCreateSphere(radius, mass);
}
void pxBody::CreateCapsule(float height = 1, float radius = 1, float mass = 10)
{
	_body = pxBodyCreateCapsule(height, radius, mass);
}
void pxBody::CreateCylinder(float radius, float height, int nbEdge, float mass = 10)
{
	_body = pxBodyCreateCylinder(radius, height, nbEdge, mass);
}
void pxBody::CreateHull(float* vbank, int nvert, float mass = 10)
{
	_body = pxBodyCreateHull(vbank, nvert, mass);
}


void pxBody::CreateTriMesh(float* vbank, int* fbank, int MESH_NBVERTICES, int MESH_NBFACES, float mass = 10)
{
	_body = pxCreateTriMesh(vbank, fbank, MESH_NBVERTICES, MESH_NBFACES, mass);
}
void pxBody::CreateTerrain(int nSize, int bank, float scale_x = 1, float scale_y = 1, float scale_z= 1)
{
	_body = pxCreateTerrain(nSize, bank, scale_x, scale_y, scale_z);
}
void pxBody::CreateTerrainFromMesh(int vbank, int fbank, int MESH_NBVERTICES, int MESH_NBFACES, float axis = 1)
{
	_body = pxCreateTerrainFromMesh(vbank, fbank, MESH_NBVERTICES, MESH_NBFACES, axis);
}
void pxBody::CreateTriMeshToFile(int vbank, int fbank, int MESH_NBVERTICES, int MESH_NBFACES, char* fname)
{
	_body = pxCreateTriMeshToFile(vbank, fbank, MESH_NBVERTICES, MESH_NBFACES, fname);
}
void pxBody::CreateTriMeshFromFile(char* fname, float mass = 10)
{
	_body = pxCreateTriMeshFromFile(fname, mass);
}





// ==================== position ============================

void pxBody::SetPosition(float pos_x, float pos_y, float pos_z)
{
	pxBodySetPosition(_body, pos_x, pos_y, pos_z);
}
void pxBody::SetRotation(float pitch, float yaw, float roll)
{
	pxBodySetRotation(_body, pitch, yaw, roll);
}

Vector3 pxBody::GetPosition()
{
	return Vector3 (pxBodyGetPositionX(_body), pxBodyGetPositionY(_body), pxBodyGetPositionZ(_body));
}
Vector3 pxBody::GetRotation()
{
	return Vector3 (pxBodyGetRotationPitch(_body), pxBodyGetRotationYaw(_body), pxBodyGetRotationRoll(_body));
}



// ==================== Flags ============================

void pxBody::SetFrozen(int stat)
{
	pxBodySetFrozen(_body, stat);
}
void pxBody::SetFrozenRot(int x, int y, int z)
{
	pxBodySetFrozenRotX(_body, x);
	pxBodySetFrozenRotY(_body, y);
	pxBodySetFrozenRotZ(_body, z);
}
void pxBody::SetFrozenPos(int x, int y, int z)
{
	pxBodySetFrozenPosX(_body, x);
	pxBodySetFrozenPosY(_body, y);
	pxBodySetFrozenPosZ(_body, z);
}
void pxBody::FlagGravity(int stat)
{
	pxBodySetFlagGravity(_body, stat);
}
void pxBody::FlagCollision(int stat)
{
	pxBodySetFlagCollision(_body, stat);
}
void pxBody::FlagResponse(int stat)
{
	pxBodySetFlagResponse(_body, stat);
}
void pxBody::FlagContacttable(int stat)
{
	pxBodySetFlagContacttable(_body, stat);
}
void pxBody::FlagMagniteble(int stat)
{
	pxBodySetFlagMagniteble(_body, stat);
}
void pxBody::FlagTriggertable(int stat)
{
	pxBodySetFlagTriggertable(_body, stat);
}
void pxBody::FlagRayCast(int stat)
{
	pxBodySetFlagRayCast(_body, stat);
}


//-------------- forces ----------------
void pxBody::AddForce(Vector3 vec, int mode = 0)
{
	pxBodyAddForce(_body, vec.x, vec.y, vec.z, mode);
}
void pxBody::AddForceAtPos(Vector3 vec, Vector3 point, int mode = 0)
{
	pxBodyAddForceAtPos(_body, vec.x, vec.y, vec.z, point.x, point.y, point.z, mode);
}
void pxBody::AddForceAtLocalPos(Vector3 vec, Vector3 point, int mode = 0)
{
	pxBodyAddForceAtLocalPos(_body, vec.x, vec.y, vec.z, point.x, point.y, point.z, mode);
}
void pxBody::AddLocalForce(Vector3 vec, int mode = 0)
{
	pxBodyAddLocalForce(_body, vec.x, vec.y, vec.z, mode);
}
void pxBody::AddLocalForceAtPos(Vector3 vec, Vector3 point, int mode = 0)
{
	pxBodyAddLocalForceAtPos(_body, vec.x, vec.y, vec.z, point.x, point.y, point.z, mode);
}
void pxBody::AddLocalForceAtLocalPos(Vector3 vec, Vector3 point, int mode = 0)
{
	pxBodyAddLocalForceAtLocalPos(_body, vec.x, vec.y, vec.z, point.x, point.y, point.z, mode);
}


void pxBody::AddTorque(Vector3 torq, int mode = 0)
{
	pxBodyAddTorque(_body, torq.x, torq.y, torq.z, mode);
}
void pxBody::AddLocalTorque(Vector3 torq, int mode = 0)
{
	pxBodyAddLocalTorque(_body, torq.x, torq.y, torq.z, mode);
}
void pxBody::SetAngularSpeed(Vector3 speed)
{
	pxBodySetAngularSpeed(_body, speed.x, speed.y, speed.z);
}
void pxBody::SetLinearSpeed(Vector3 speed)
{
	pxBodySetLinearSpeed(_body, speed.x, speed.y, speed.z);
}
void pxBody::SetLocalLinearSpeed(Vector3 speed)
{
	pxBodySetLocalLinearSpeed(_body, speed.x, speed.y, speed.z);
}
void pxBody::SetLocalAngularSpeed(Vector3 speed)
{
	pxBodySetLocalAngularSpeed(_body, speed.x, speed.y, speed.z);
}
void pxBody::SetAngularDamping(float angDamp)
{
	pxBodySetAngularDamping(_body, angDamp);
}
void pxBody::SetLinearDamping(float linDamp)
{
	pxBodySetLinearDamping(_body, linDamp);
}
void pxBody::SetAngularMomentum(Vector3 mom)
{
	pxBodySetAngularMomentum(_body, mom.x, mom.y, mom.z);
}
void pxBody::SetLinearMomentum(Vector3 mom)
{
	pxBodySetLinearMomentum(_body, mom.x, mom.y, mom.z);
}
void pxBody::SetMaxAngularSpeed(float speed)
{
	pxBodySetMaxAngularSpeed(_body, speed);
}
void pxBody::SetSleepEnergyThreshold(float tres)
{
	pxBodySetSleepEnergyThreshold(_body, tres);
}
void pxBody::SetSleepAngularVelocity(float tres)
{
	pxBodySetSleepAngularVelocity(_body, tres);
}
void pxBody::SetSleepLinearVelocity(float tres)
{
	pxBodySetSleepLinearVelocity(_body, tres);
}
void pxBody::SetSleepWakeUp(float wakeCounterValue)
{
	pxBodySetSleepWakeUp(_body, wakeCounterValue);
}
void pxBody::SetSleepPut()
{
	pxBodySetSleepPut(_body);
}
void pxBody::SetSolverIterationCount(int itercount)
{
	pxBodySetSolverIterationCount(_body, itercount);
}
void pxBody::SetBodyName(char* name)
{
	pxBodySetBodyName(_body, name);
}
void pxBody::SetBodyEntity(int entity)
{
	pxBodySetBodyEntity(_body, entity);
}
void pxBody::SetBodyUserData(int userdata)
{
	pxBodySetBodyUserData(_body, userdata);
}
void pxBody::MoveBodyToPoint(float maxspeed, float x, float y, float z)
{
	pxMoveBodyToPoint(_body, maxspeed, x, y, z);
}
void pxBody::AddBodytoBody(int body)
{
	pxAddBodytoBody(_body, body);
}
int pxBody::CopyBody()
{
	return pxCopyBody(_body);
}



float pxBody::GetAngularSpeedModul()
{
	return pxBodyGetAngularSpeed(_body);
}
Vector3 pxBody::GetAngularSpeed()
{
	return Vector3(pxBodyGetAngularSpeedX(_body), pxBodyGetAngularSpeedY(_body), pxBodyGetAngularSpeedZ(_body));
}
Vector3 pxBody::GetLocalAngularSpeed()
{
	return Vector3(pxBodyGetLocalAngularSpeedX(_body), pxBodyGetLocalAngularSpeedY(_body), pxBodyGetLocalAngularSpeedZ(_body));
}
float pxBody::GetLinearSpeedModul()
{
	return pxBodyGetLinearSpeed(_body);
}
Vector3 pxBody::GetLinearSpeed()
{
	return Vector3(pxBodyGetLinearSpeedX(_body), pxBodyGetLinearSpeedY(_body), pxBodyGetLinearSpeedZ(_body));
}
Vector3 pxBody::GetLocalLinearSpeed()
{
	return Vector3(pxBodyGetLocalLinearSpeedX(_body), pxBodyGetLocalLinearSpeedY(_body), pxBodyGetLocalLinearSpeedZ(_body));
}
Vector3 pxBody::GetLinearVecSpeed()
{
	return Vector3(pxBodyGetLinearVecSpeedX(_body), pxBodyGetLinearVecSpeedY(_body), pxBodyGetLinearVecSpeedZ(_body));
}
float pxBody::GetLocalPointSpeedModul(Vector3 point)
{
	return pxBodyGetLocalPointSpeed(_body, point.x, point.y, point.z);
}
Vector3 pxBody::GetLocalPointSpeed(Vector3 point)
{
	return Vector3(pxBodyGetLocalPointSpeedX(_body, point.x, point.y, point.z), pxBodyGetLocalPointSpeedY(_body, point.x, point.y, point.z), pxBodyGetLocalPointSpeedZ(_body, point.x, point.y, point.z));
}



// ----------- rasnoe ----------------------------

float pxBody::GetAngularDamping()
{
	return pxBodyGetAngularDamping(_body);
}
float pxBody::GetLinearDamping()
{
	return pxBodyGetLinearDamping(_body);
}
float pxBody::GetAngularMomentum()
{
	return pxBodyGetAngularMomentum(_body);
}
float pxBody::GetLinearMomentum()
{
	return pxBodyGetLinearMomentum(_body);
}
float pxBody::GetMaxAngularVelocity()
{
	return pxBodyGetMaxAngularVelocity(_body);
}
char* pxBody::GetName()
{
	return pxGetBodyName(_body);
}
int   pxBody::GetEntity()
{
	return pxGetBodyEntity(_body);
}
int   pxBody::GetUserData()
{
	return pxGetBodyUserData(_body);
}
float pxBody::GetSleepAngularVelocity()
{
	return pxBodyGetSleepAngularVelocity(_body);
}
float pxBody::GetSleepLinearVelocity()
{
	return pxBodyGetSleepLinearVelocity(_body);
}
int   pxBody::IsDynamic()
{
	return pxBodyIsDynamic(_body);
}
int   pxBody::IsSleeping()
{
	return pxBodyIsSleeping(_body);
}


//------------------- mass -----------------------

void  pxBody::SetMass(float mass)
{
	pxBodySetMass(_body, mass);
}
void  pxBody::SetCMassLocalPosition(Vector3 pos)
{
	pxBodySetCMassLocalPosition(_body, pos.x, pos.y, pos.z);
}
void  pxBody::SetCMassGlobalPosition(Vector3 pos)
{
	pxBodySetCMassGlobalPosition(_body, pos.x, pos.y, pos.z);
}
void  pxBody::SetMassSpaceInertiaTensor(Vector3 pos)
{
	pxBodySetMassSpaceInertiaTensor(_body, pos.x, pos.y, pos.z);
}
Vector3 pxBody::GetCMassGlobalPosition()
{
	return Vector3(pxBodyGetCMassGlobalPositionX(_body), pxBodyGetCMassGlobalPositionY(_body), pxBodyGetCMassGlobalPositionZ(_body));
}
Vector3 pxBody::GetCMassLocalPosition()
{
	return Vector3(pxBodyGetCMassLocalPositionX(_body), pxBodyGetCMassLocalPositionY(_body), pxBodyGetCMassLocalPositionZ(_body));
}
float pxBody::GetMass()
{
	return pxBodyGetMass(_body);
}



//----------------- contacts -----------------
int pxBody::Contacts()
{
	return pxGetContacts(_body);
}
int pxBody::ContactGetBody(int coll)
{
	return pxContactGetBody(_body, coll);
}
Vector3 pxBody::ContactGetPoint(int coll)
{
	return Vector3(pxContactGetPointX(_body, coll), pxContactGetPointY(_body, coll), pxContactGetPointZ(_body, coll));
}
Vector3 pxBody::ContactGetPointN(int coll)
{
	return Vector3(pxContactGetPointNX(_body, coll), pxContactGetPointNY(_body, coll), pxContactGetPointNZ(_body, coll));
}
float pxBody::ContactGetForceNModul(int coll)
{
	return pxContactGetForceN(_body, coll);
}
Vector3 pxBody::ContactGetForceN(int coll)
{
	return Vector3(pxContactGetForceNX(_body, coll), pxContactGetForceNY(_body, coll), pxContactGetForceNZ(_body, coll));
}
float pxBody::ContactGetForceTModul(int coll)
{
	return pxContactGetForceT(_body, coll);
}
Vector3 pxBody::ContactGetForceT(int coll)
{
	return Vector3(pxContactGetForceTX(_body, coll), pxContactGetForceTY(_body, coll), pxContactGetForceTZ(_body, coll));
}
int pxBody::ContactEventsOnStartTouch()
{
	return pxContactEventsOnStartTouch(_body);
}
int pxBody::ContactEventsOnEndTouch()
{
	return pxContactEventsOnEndTouch(_body);
}


#endif