#ifndef _PX_JOINT_H_
#define _PX_JOINT_H_

class pxJoint
{
public:
	int _joint;
public:
	pxJoint();
	void Delete();
	void CreateHinge(int body1, int body2, Vector3 pos, Vector3 norm);
	void CreateSpherical(int body1, int body2, Vector3 pos, Vector3 norm);
	void CreateDistance(int body1, int body2, Vector3 p1, Vector3 p2);
	void CreateFixed(int body1, int body2);
	void CreateCylindrical(int body1, int body2, Vector3 pos, Vector3 norm, float min_limit, float max_limit);
	void CreatePrismatic(int body1, int body2, Vector3 pos, Vector3 norm, float min_limit, float max_limit);
	void CreateOnLine(int body1, Vector3 pos, Vector3 norm);
	void CreateInPlane(int body1, Vector3 pos, Vector3 norm);
	void CreatePulley(int body1, int body2, float distance, float stiff, float k_ratio);

	void CreateD6Joint(int body1, int body2, Vector3 pos, Vector3 norm);
	void SetPosMotion(int xMot, int yMot, int zMot);
	void SetAngleMotion(int twistMot, int swing1Mot, int swing2Mot);
	void SetLinearLimit(float lim);
	void SetSwingLimit(float swing1Lim, float swing2Lim);
	void SetTwistLimit(float lowLim, float heighLim);

	void HingeSetLimit(float min, float max);
	void HingeSetSpring(float spr, float targetVal);
	void SphericalSetLimitAngle(float angle, float hardn, float restit);
	void SphericalSetLimitTwist(float mintwist, float maxtwist, float spr, float damp, float targetVal);
	void SphericalSetLimitSpring(float spr, float damp, float targetVal);
	void DistanceSetPoint(float mindist, float maxdist);
	void DistanceSetSpring(float spr, float damp);
	void AddLimitPlane(Vector3 point, Vector3 norm);
	void PulleySetAnchor(Vector3 point1, Vector3 point2);
	void SetLocalAttachBody(Vector3 point1, Vector3 point2);
	void HingeSetMotor(float force, float velTarget);
	void SetBreakable(float force, float torque);
	int IsBroken();
	void HingeSetCollision();
	void SphericalSetCollision();
	void DistanceSetCollision();
	void CylindricalSetCollision();
};

pxJoint::pxJoint()
{
	_joint = 0;
}

void pxJoint::Delete()
{
	 pxDeleteJoint(_joint);
}

void pxJoint::CreateHinge(int body1, int body2, Vector3 pos, Vector3 norm)
{
	  _joint = pxJointCreateHinge(body1, body2, pos.x, pos.y, pos.z, norm.x, norm.y, norm.z);
}
void pxJoint::CreateSpherical(int body1, int body2, Vector3 pos, Vector3 norm)
{
	  _joint = pxJointCreateSpherical(body1, body2, pos.x, pos.y, pos.z, norm.x, norm.y, norm.z);
}
void pxJoint::CreateDistance(int body1, int body2, Vector3 p1, Vector3 p2)
{
	  _joint = pxJointCreateDistance(body1, body2, p1.x, p1.y, p1.z, p2.x, p2.y, p2.z);
}
void pxJoint::CreateFixed(int body1, int body2)
{
	 _joint =  pxJointCreateFixed(body1, body2);
}
void pxJoint::CreateCylindrical(int body1, int body2, Vector3 pos, Vector3 norm, float min_limit, float max_limit)
{
	  _joint = pxJointCreateCylindrical(body1, body2, pos.x, pos.y, pos.z, norm.x, norm.y, norm.z, min_limit, max_limit);
}
void pxJoint::CreatePrismatic(int body1, int body2, Vector3 pos, Vector3 norm, float min_limit, float max_limit)
{
	  _joint = pxJointCreatePrismatic(body1, body2, pos.x, pos.y, pos.z, norm.x, norm.y, norm.z, min_limit, max_limit);
}
void pxJoint::CreateOnLine(int body1, Vector3 pos, Vector3 norm)
{
	  _joint = pxJointCreateOnLine(body1, pos.x, pos.y, pos.z, norm.x, norm.y, norm.z);
}
void pxJoint::CreateInPlane(int body1, Vector3 pos, Vector3 norm)
{
	  _joint = pxJointCreateInPlane(body1, pos.x, pos.y, pos.z, norm.x, norm.y, norm.z);
}
void pxJoint::CreatePulley(int body1, int body2, float distance, float stiff, float k_ratio)
{
	 _joint =  pxJointCreatePulley(body1, body2, distance, stiff, k_ratio);
}


//--------------- D6 joint --------------------------
void pxJoint::CreateD6Joint(int body1, int body2, Vector3 pos, Vector3 norm)
{
	_joint = pxJointCreateD6Joint(body1, body2, pos.x, pos.y, pos.z, norm.x, norm.y, norm.z);
}
void pxJoint::SetPosMotion(int xMot, int yMot, int zMot)
{
	pxD6JointSetPosMotion(_joint, xMot, yMot, zMot);
}
void pxJoint::SetAngleMotion(int twistMot, int swing1Mot, int swing2Mot)
{
	pxD6JointSetAngleMotion(_joint, twistMot, swing1Mot, swing2Mot);
}
void pxJoint::SetLinearLimit(float lim)
{
	pxD6JointSetLinearLimit(_joint, lim);
}
void pxJoint::SetSwingLimit(float swing1Lim, float swing2Lim)
{
	pxD6JointSetSwingLimit(_joint, swing1Lim, swing2Lim);
}
void pxJoint::SetTwistLimit(float lowLim, float heighLim)
{
	pxD6JointSetTwistLimit(_joint, lowLim, heighLim);
}


//----------------- parameters ----------------

void pxJoint::HingeSetLimit(float min, float max)
{
	pxJointHingeSetLimit(_joint, min, max);
}
void pxJoint::HingeSetSpring(float spr, float targetVal)
{
	pxJointHingeSetSpring(_joint, spr, targetVal);
}
void pxJoint::SphericalSetLimitAngle(float angle, float hardn, float restit)
{
	pxJointSphericalSetLimitAngle(_joint, angle, hardn, restit);
}
void pxJoint::SphericalSetLimitTwist(float mintwist, float maxtwist, float spr, float damp, float targetVal)
{
	pxJointSphericalSetLimitTwist(_joint, mintwist, maxtwist, spr, damp, targetVal);
}
void pxJoint::SphericalSetLimitSpring(float spr, float damp, float targetVal)
{
	pxJointSphericalSetLimitSpring(_joint, spr, damp, targetVal);
}
void pxJoint::DistanceSetPoint(float mindist, float maxdist)
{
	pxJointDistanceSetPoint(_joint, mindist, maxdist);
}
void pxJoint::DistanceSetSpring(float spr, float damp)
{
	pxJointDistanceSetSpring(_joint, spr, damp);
}
void pxJoint::AddLimitPlane(Vector3 point, Vector3 norm)
{
	pxJointAddLimitPlane(_joint, point.x, point.y, point.z, norm.x, norm.y, norm.z);
}
void pxJoint::PulleySetAnchor(Vector3 point1, Vector3 point2)
{
	pxJointPulleySetAnchor(_joint, point1.x, point1.y, point1.z, point2.x, point2.y, point2.z);
}
void pxJoint::SetLocalAttachBody(Vector3 point1, Vector3 point2)
{
	pxJointPulleySetLocalAttachBody(_joint, point1.x, point1.y, point1.z, point2.x, point2.y, point2.z);
}
void pxJoint::HingeSetMotor(float force, float velTarget)
{
	pxJointHingeSetMotor(_joint, force, velTarget);
}
void pxJoint::SetBreakable(float force, float torque)
{
	pxJointSetBreakable(_joint, force, torque);
}
int pxJoint::IsBroken()
{
	return pxJointIsBroken(_joint);
}
void pxJoint::HingeSetCollision()
{
	pxJointHingeSetCollision(_joint);
}
void pxJoint::SphericalSetCollision()
{
	pxJointSphericalSetCollision(_joint);
}
void pxJoint::DistanceSetCollision()
{
	pxJointDistanceSetCollision(_joint);
}
void pxJoint::CylindricalSetCollision()
{
	pxJointCylindricalSetCollision(_joint);
}

#endif